/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef KIVIO_RULER_H
#define KIVIO_RULER_H

#include "tkunits.h"

#include <qframe.h>
#include <qpainter.h>
#include <qpixmap.h>

class KivioRuler : public QFrame
{ Q_OBJECT
public:
  enum Orientation { Horizontal, Vertical };

  KivioRuler(Orientation o, MeasurementUnit mu = UnitPoint, QWidget* parent = 0L, const char* name = 0L);
  ~KivioRuler();

  MeasurementUnit measurementUnit () const;
  void setMeasurementUnit (MeasurementUnit mu);

public slots:
  void setZoom( int zoom );
  void updatePointer( int x, int y );
  void updateVisibleArea( int xpos, int ypos );

  void hide();
  void show();

signals:
  /*emit signal for drawing a ruler, note: the position is in sceen positions
    and might be out of the drawing area; in such a case the position should
    be ignored by the slot of drawKivioRuler and -- if drawn -- remove the
    draw'ed-KivioRuler; note: the can be only one drawKivioRuler at a time
    orientationHoriz = true <=> horizontal
   */
  void drawHelpline(int x, int y, bool orientationHoriz);
  void addHelpline (int x, int y, bool orientationHoriz);

protected:
  void paintEvent  (QPaintEvent *e);
  void resizeEvent (QResizeEvent *e);

  void recalculateSize (QResizeEvent *e);
  void drawKivioRuler ();

  void initMarker (int w, int h);
  void drawNums( QPainter*, int x, int y, QString& num, bool orientationHoriz );

protected slots:
  void mousePressEvent ( QMouseEvent * );
  void mouseReleaseEvent ( QMouseEvent * );
  void mouseMoveEvent ( QMouseEvent * );

private:
  bool isMousePressed;
  int m_pZoom;
  MeasurementUnit munit;
  Orientation orientation;
  QPixmap *buffer;
  int firstVisible;
  int currentPosition;
  QPixmap* marker;
  QPixmap* bg;
  QPixmap* m_pNums;
};

#endif
