/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2000 theKompany.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "tool_zoom.h"
#include "kivio_view.h"
#include "kivio_view.h"
#include "kivio_page.h"
#include "kivio_canvas.h"
#include "tooldockbase.h"
#include "tooldockmanager.h"

#include "toolbarseparator.h"

#include <klocale.h>
#include <ktoolbar.h>
#include <kaction.h>
#include <kiconloader.h>

#include <qapplication.h>
#include <qcursor.h>

ZoomTool::ZoomTool( KivioView* view )
:Tool(view)
{
  setSortNum(1);
  m_pToolBar = 0L;

  new ToolBarSeparator(actionCollection(),"---");

  m_z1 = new KRadioAction( i18n("Zoom"), "kivio_zoom", Key_F2, actionCollection(), "zoom" );
  m_z2 = new KRadioAction( i18n("Hand"), "kivio_zoom_hand", Key_F3, actionCollection(), "zoomHand" );
  m_z1->setExclusiveGroup("zoomAction");
  m_z2->setExclusiveGroup("zoomAction");
  connect(m_z1,SIGNAL(activated()),SLOT(zoomActivated()));
  connect(m_z2,SIGNAL(activated()),SLOT(handActivated()));

  m_pPlus = new KAction( i18n("Zoom plus"), "kivio_zoom_plus", SHIFT+Key_F2, actionCollection(), "zoomPlus" );
  connect(m_pPlus,SIGNAL(activated()),SLOT(zoomPlus()));

  m_pMinus = new KAction( i18n("Zoom minus"), "kivio_zoom_minus", SHIFT+Key_F3, actionCollection(), "zoomMinus" );
  connect(m_pMinus,SIGNAL(activated()),SLOT(zoomMinus()));

  KAction* m_3 = new KAction( i18n("Zoom width"), "kivio_zoom_width", SHIFT+Key_F4, actionCollection(), "zoomWidth" );
  connect(m_3,SIGNAL(activated()),SLOT(zoomWidth()));

  KAction* m_4 = new KAction( i18n("Zoom height"), "kivio_zoom_height", SHIFT+Key_F5, actionCollection(), "zoomHeight" );
  connect(m_4,SIGNAL(activated()),SLOT(zoomHeight()));

  KAction* m_5 = new KAction( i18n("Zoom page"), "kivio_zoom_page", SHIFT+Key_F6, actionCollection(), "zoomPage" );
  connect(m_5,SIGNAL(activated()),SLOT(zoomPage()));

  KAction* m_6 = new KAction( i18n("Zoom selected"), "kivio_zoom_selected", CTRL+Key_Y, actionCollection(), "zoomSelected" );
  connect(m_6,SIGNAL(activated()),SLOT(zoomSelected()));

  KAction* m_7 = new KAction( i18n("Zoom all objects"), "kivio_zoom_allobject", 0, actionCollection(), "zoomAllObjects" );
  connect(m_7,SIGNAL(activated()),SLOT(zoomAllobjects()));

  KToggleAction* m_8 = new KToggleAction( i18n("Zoom manager"), "kivio_zoom_manager", 0, actionCollection(), "zoomManager" );

  QPixmap pix;

  pix = BarIcon("kivio_zoom_plus");
  m_pPlusCursor = new QCursor(pix,pix.width()/2,pix.height()/2);

  pix = BarIcon("kivio_zoom_minus");
  m_pMinusCursor = new QCursor(pix,pix.width()/2,pix.height()/2);

  pix = BarIcon("kivio_zoom_hand");
  m_handCursor = new QCursor(pix,pix.width()/2,pix.height()/2);

  ToolSelectAction* zoom = new ToolSelectAction( actionCollection(), "ToolAction" );
  zoom->insert(m_z1);
  zoom->insert(m_z2);

  QWidget* zoomManagerWidget = new QWidget();
  zoomManagerWidget->setMinimumSize(200,300);
  m_pZoomManagerBase = view->toolDockManager()->createToolDock(zoomManagerWidget,i18n("Zoom Manager"));
  m_pZoomManagerBase->move(0,0);


  connect( m_8,
           SIGNAL(toggled(bool)),
           m_pZoomManagerBase,
           SLOT(makeVisible(bool)));

  connect( m_pZoomManagerBase,
           SIGNAL(visibleChange(bool)),
           SLOT(toggleShowManager(bool)));
}

ZoomTool::~ZoomTool()
{
  delete m_pPlusCursor;
  delete m_pMinusCursor;
  delete m_handCursor;
}

void ZoomTool::processEvent( QEvent* e )
{
  if ( !m_bHandMode )
    switch (e->type()) {
      case QEvent::KeyPress:
        if (!m_bLockKeyboard) {
          m_pCurrent = m_pMinus;
          m_pCanvas->setCursor(*m_pMinusCursor);
        }
        break;
      case QEvent::KeyRelease:
        if (!m_bLockKeyboard) {
          m_pCurrent = m_pPlus;
          m_pCanvas->setCursor(*m_pPlusCursor);
        }
        break;
      case QEvent::MouseButtonPress:
        if ( m_pCurrent == m_pMinus )
          m_pCurrent->activate();
        else {
          m_bLockKeyboard = true;
          m_bDrawRubber = true;
      		m_pCanvas->startRectDraw( ((QMouseEvent*)e)->pos(), KivioCanvas::Rubber );
        }
        break;
      case QEvent::MouseButtonRelease:
  	    m_pCanvas->endRectDraw();
        m_bDrawRubber = false;
        m_bLockKeyboard = false;
        zoomRect(m_pCanvas->rect());
        break;
      case QEvent::MouseMove:
  	    if (m_bDrawRubber)
    	    m_pCanvas->continueRectDraw( ((QMouseEvent*)e)->pos(), KivioCanvas::Rubber );
        break;
      default:
        break;
    }
  else
    switch (e->type()) {
      case QEvent::MouseButtonPress:
        isHandMousePressed = true;
        m_pCanvas->setUpdatesEnabled(false);
        mousePos = ((QMouseEvent*)e)->pos();
        break;
      case QEvent::MouseButtonRelease:
        isHandMousePressed = false;
        m_pCanvas->setUpdatesEnabled(true);
        break;
      case QEvent::MouseMove:
        if (isHandMousePressed) {
          QPoint newPos = ((QMouseEvent*)e)->pos();
          mousePos -= newPos;
          m_pCanvas->scrollDx(-mousePos.x());
          m_pCanvas->scrollDy(-mousePos.y());
          mousePos = newPos;
        }
        break;
      default:
        break;
    }
}

void ZoomTool::activateGUI( KXMLGUIFactory* factory )
{
  m_pToolBar = (KToolBar*)factory->container("ZoomToolBar",this);
  if (m_pToolBar)
    m_pToolBar->hide();
}

void ZoomTool::deactivateGUI( KXMLGUIFactory* )
{
}

void ZoomTool::activate()
{
  m_pToolBar = (KToolBar*)m_pView->factory()->container("ZoomToolBar",this);
  if (m_pToolBar)
    m_pToolBar->show();

  m_pCurrent = m_pPlus;
}

void ZoomTool::deactivate()
{
  debug("ZoomTool DeActivate");
  if (m_pToolBar)
   m_pToolBar->hide();

  m_pCurrent = 0L;

  m_z1->setChecked(false);
  m_z2->setChecked(false);

  if (!m_pCanvas->isUpdatesEnabled()) {
    m_pCanvas->setUpdatesEnabled(true);
  }
}

void ZoomTool::configure()
{
}

void ZoomTool::zoomActivated()
{
  debug("Zoom Activate");
  setOverride();
  m_pCanvas->setCursor(*m_pPlusCursor);
  m_bHandMode = false;
  m_bDrawRubber = false;
  m_bLockKeyboard = false;
}

void ZoomTool::handActivated()
{
  debug("Hand Activate");
  setOverride();
  m_pCanvas->setCursor(*m_handCursor);
  m_bHandMode = true;
  isHandMousePressed = false;
}

void ZoomTool::zoomPlus()
{
  setOverride();
  m_pCanvas->zoomIn();
  removeOverride();
}

void ZoomTool::zoomMinus()
{
  setOverride();
  m_pCanvas->zoomOut();
  removeOverride();
}

void ZoomTool::zoomWidth()
{
  setOverride();

  KoPageLayout pl = m_pCanvas->activePage()->paperLayout();

  int cw = QMAX(10,m_pCanvas->width()-20);
  float w = cvtMmToPt(pl.mmWidth);
  int zw = (int)(cw/w*100.0);

  m_pCanvas->setUpdatesEnabled(false);
  m_pCanvas->centerPage();
  m_pCanvas->setZoom(zw);
  m_pCanvas->setUpdatesEnabled(true);

  removeOverride();
}

void ZoomTool::zoomHeight()
{
  setOverride();

  KoPageLayout pl = m_pCanvas->activePage()->paperLayout();

  int ch = QMAX(10,m_pCanvas->height()-20);
  float h = cvtMmToPt(pl.mmHeight);

  int zh = (int)(ch/h*100.0);

  m_pCanvas->setUpdatesEnabled(false);
  m_pCanvas->centerPage();
  m_pCanvas->setZoom(zh);
  m_pCanvas->setUpdatesEnabled(true);

  removeOverride();
}

void ZoomTool::zoomPage()
{
  setOverride();

  KoPageLayout pl = m_pCanvas->activePage()->paperLayout();

  int cw = QMAX(10,m_pCanvas->width()-20);
  int ch = QMAX(10,m_pCanvas->height()-20);
  float w = cvtMmToPt(pl.mmWidth);
  float h = cvtMmToPt(pl.mmHeight);

  int zw = (int)(cw/w*100.0);
  int zh = (int)(ch/h*100.0);

  m_pCanvas->setUpdatesEnabled(false);
  m_pCanvas->centerPage();
  m_pCanvas->setZoom(QMIN(zw,zh));
  m_pCanvas->setUpdatesEnabled(true);

  removeOverride();
}

void ZoomTool::zoomSelected()
{
  debug("ZoomSelected Activate");
  setOverride();
  removeOverride();
}

void ZoomTool::zoomAllobjects()
{
  setOverride();
  removeOverride();
}

void ZoomTool::toggleShowManager( bool isShow )
{
  ((KToggleAction*)actionCollection()->action("zoomManager"))->setChecked(isShow);
}

void ZoomTool::zoomRect( QRect r )
{
  if (r.isEmpty()) {
    m_pCanvas->zoomIn();
    return;
  }

  TKPoint p0 = m_pCanvas->mapFromScreen(r.topLeft());

  m_pCanvas->setUpdatesEnabled(false);

  int cw = m_pCanvas->width();
  int ch = m_pCanvas->height();
  int cz = m_pCanvas->zoom();

  int zw = cz*cw/r.width();
  int zh = cz*ch/r.height();
  int z = QMIN(zw,zh);
  m_pCanvas->setZoom(z);

  m_pCanvas->setUpdatesEnabled(true);

  QPoint p = m_pCanvas->mapToScreen(p0);

  // center zoomed rect
  p -= QPoint( (cw - r.width()*(z/cz))/2, (ch - r.height()*(z/cz))/2 );

  m_pCanvas->scrollDx(-p.x());
  m_pCanvas->scrollDy(-p.y());
}
